import logging
import os
import sys
import threading
from datetime import datetime

import pygame

from overcooked_simulator import ROOT_DIR
from overcooked_simulator.gui_2d_vis.overcooked_gui import PyGameGUI
from overcooked_simulator.simulation_runner import Simulator

log = logging.getLogger(__name__)


def setup_logging():
    path_logs = ROOT_DIR.parent / "logs"
    os.makedirs(path_logs, exist_ok=True)
    logging.basicConfig(
        level=logging.DEBUG,
        format="%(asctime)s %(levelname)-8s %(name)-50s %(message)s",
        handlers=[
            logging.FileHandler(
                path_logs / f"{datetime.now().strftime('%Y-%m-%d_%H-%M-%S')}_debug.log",
                encoding="utf-8",
            ),
            logging.StreamHandler(sys.stdout),
        ],
    )
    logging.getLogger("matplotlib").setLevel(logging.WARNING)


def main():
    simulator = Simulator(
        ROOT_DIR / "game_content" / "environment_config.yaml",
        ROOT_DIR / "game_content" / "layouts" / "basic.layout",
        600,
    )
    number_player = 2
    for i in range(number_player):
        player_name = f"p{i}"
        simulator.register_player(player_name)

    # TODO maybe read the player names and keyboard keys from config file?
    keys1 = [
        pygame.K_LEFT,
        pygame.K_RIGHT,
        pygame.K_UP,
        pygame.K_DOWN,
        pygame.K_SPACE,
        pygame.K_i,
    ]
    keys2 = [pygame.K_a, pygame.K_d, pygame.K_w, pygame.K_s, pygame.K_f, pygame.K_e]

    gui = PyGameGUI(simulator, [f"p{i}" for i in range(number_player)], [keys1, keys2])
    gui.start_pygame()
    sys.exit()


if __name__ == "__main__":
    setup_logging()
    try:
        main()
    except Exception as e:
        log.exception(e)
        for thread in threading.enumerate():
            if isinstance(thread, Simulator):
                thread.stop()
                thread.join()
        sys.exit(1)
